/* compile: g++ -Wall -fexceptions -c Process.cc */
/**
 * This is a very simple implementation of a
 * process. How to use it:
 * MyProcess myprocess();
 * myprocess.start();
 * here goes the parent...
 * wait(&status);
 *
 * and you have to extend Process and override
 * the run() method which does nothing
 * if the process creation does not work then
 * an the exception ForkException is raised
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <string.h>
#include "Process.h"

int Process::start() {
  //we create a process
  int pid=fork();
  switch(pid) {
  case -1: //error
    throw ForkException(errno);
    break;
  case 0: //in the child
    run();
    exit(0);
  default: //in the parent
    return pid;
  }
}

ForkException::ForkException(int error)
:message(strerror(error))
{}

ForkException::~ForkException() {
  delete message;
}

char* ForkException::getMessage() {
  return message;
}

ostream& operator << (ostream& out,const ForkException& fe) {
  out<<fe.message;
  return out;
}
